@@ -43,7 +43,7 @@
 #endif
 #include <gssapi.h>
 
-RCSID("$Id: gssapi.c,v 1.3 1998/05/12 12:27:07 joda Exp $");
+RCSID("$Id: gssapi.c,v 1.5 1998/11/21 22:12:59 assar Exp $");
 
 struct gss_data {
     gss_ctx_id_t context_hdl;
@@ -183,12 +183,12 @@ gss_adat(void *app_data, void *buf, size_t len)
 	    reply(235, "ADAT Complete");
 	sec_complete = 1;
 
-    } else if(maj_stat == GSS_S_CONTINUE_NEEDED)
+    } else if(maj_stat == GSS_S_CONTINUE_NEEDED) {
 	if(p)
 	    reply(335, "ADAT=%s", p);
 	else
 	    reply(335, "OK, need more data");
-    else
+    } else
 	reply(535, "foo?");
 out:
     free(p);
@@ -246,10 +246,10 @@ gss_auth(void *app_data, char *host)
     bindings = malloc(sizeof(*bindings));
     bindings->initiator_addrtype = GSS_C_AF_INET;
     bindings->initiator_address.length = 4;
-    bindings->initiator_address.value = &hisctladdr.sin_addr;
+    bindings->initiator_address.value = &myctladdr.sin_addr;
     bindings->acceptor_addrtype = GSS_C_AF_INET;
     bindings->acceptor_address.length = 4;
-    bindings->acceptor_address.value = &myctladdr.sin_addr;
+    bindings->acceptor_address.value = &hisctladdr.sin_addr;
     bindings->application_data.length = 0;
     bindings->application_data.value = NULL;
 
