@@ -55,7 +55,7 @@
 #include <config.h>
 #endif
 
-RCSID("$Id: kerberos.c,v 1.31 1997/05/20 18:40:38 bg Exp $");
+RCSID("$Id: kerberos.c,v 1.44 1999/02/11 21:02:43 joda Exp $");
 
 #ifdef	KRB4
 #ifdef HAVE_SYS_TYPES_H
@@ -70,11 +70,11 @@ RCSID("$Id: kerberos.c,v 1.31 1997/05/20 18:40:38 bg Exp $");
 #include <pwd.h>
 #include <stdlib.h>
 #include <string.h>
+#include <roken.h>
 #ifdef SOCKS
 #include <socks.h>
 #endif
 
-#include <roken.h>
 
 #include "encrypt.h"
 #include "auth.h"
@@ -102,6 +102,7 @@ static	KTEXT_ST auth;
 static	char name[ANAME_SZ];
 static	AUTH_DAT adat;
 static des_cblock session_key;
+static des_cblock cred_session;
 static des_key_schedule sched;
 static des_cblock challenge;
 static int auth_done; /* XXX */
@@ -111,13 +112,13 @@ static int unpack_cred(unsigned char *buf, int len, CREDENTIALS *cred);
 
 
 static int
-Data(Authenticator *ap, int type, void *d, int c)
+Data(Authenticator *ap, int type, const void *d, int c)
 {
     unsigned char *p = str_data + 4;
-    unsigned char *cd = (unsigned char *)d;
+    const unsigned char *cd = (const unsigned char *)d;
 
     if (c == -1)
-	c = strlen((char *)cd);
+	c = strlen((const char *)cd);
 
     if (auth_debug_mode) {
 	printf("%s:%d: [%d] (%d)",
@@ -138,7 +139,7 @@ Data(Authenticator *ap, int type, void *d, int c)
     *p++ = SE;
     if (str_data[3] == TELQUAL_IS)
 	printsub('>', &str_data[2], p - (&str_data[2]));
-    return(net_write(str_data, p - str_data));
+    return(telnet_net_write(str_data, p - str_data));
 }
 
 int
@@ -166,8 +167,6 @@ kerberos4_send(char *name, Authenticator *ap)
     KTEXT_ST auth;
     char instance[INST_SZ];
     char *realm;
-    char *krb_realmofhost(const char *);
-    char *krb_get_phost(const char *);
     CREDENTIALS cred;
     int r;
 
@@ -181,10 +180,9 @@ kerberos4_send(char *name, Authenticator *ap)
 
     memset(instance, 0, sizeof(instance));
 
-    if (realm = krb_get_phost(RemoteHostName))
-	strncpy(instance, realm, sizeof(instance));
-
-    instance[sizeof(instance)-1] = '\0';
+    strcpy_truncate (instance,
+		     krb_get_phost(RemoteHostName),
+		     INST_SZ);
 
     realm = dest_realm ? dest_realm : krb_realmofhost(RemoteHostName);
 
@@ -192,11 +190,13 @@ kerberos4_send(char *name, Authenticator *ap)
 	printf("Kerberos V4: no realm for %s\r\n", RemoteHostName);
 	return(0);
     }
-    if (r = krb_mk_req(&auth, KRB_SERVICE_NAME, instance, realm, 0L)) {
+    r = krb_mk_req(&auth, KRB_SERVICE_NAME, instance, realm, 0L);
+    if (r) {
 	printf("mk_req failed: %s\r\n", krb_get_err_text(r));
 	return(0);
     }
-    if (r = krb_get_cred(KRB_SERVICE_NAME, instance, realm, &cred)) {
+    r = krb_get_cred(KRB_SERVICE_NAME, instance, realm, &cred);
+    if (r) {
 	printf("get_cred failed: %s\r\n", krb_get_err_text(r));
 	return(0);
     }
@@ -218,6 +218,7 @@ kerberos4_send(char *name, Authenticator *ap)
 	int i;
 
 	des_key_sched(&cred.session, sched);
+	memcpy (&cred_session, &cred.session, sizeof(cred_session));
 	des_init_random_number_generator(&cred.session);
 	des_new_random_key(&session_key);
 	des_ecb_encrypt(&session_key, &session_key, sched, 0);
@@ -267,9 +268,11 @@ kerberos4_send_oneway(Authenticator *ap)
 void
 kerberos4_is(Authenticator *ap, unsigned char *data, int cnt)
 {
+    struct sockaddr_in addr;
     char realm[REALM_SZ];
     char instance[INST_SZ];
     int r;
+    int addr_len;
 
     if (cnt-- < 1)
 	return;
@@ -290,8 +293,25 @@ kerberos4_is(Authenticator *ap, unsigned char *data, int cnt)
 	    printf("\r\n");
 	}
 	k_getsockinst(0, instance, sizeof(instance));
-	if (r = krb_rd_req(&auth, KRB_SERVICE_NAME,
-			   instance, 0, &adat, "")) {
+	addr_len = sizeof(addr);
+	if(getpeername(0, (struct sockaddr *)&addr, &addr_len) < 0) {
+	    if(auth_debug_mode)
+		printf("getpeername failed\r\n");
+	    Data(ap, KRB_REJECT, "getpeername failed", -1);
+	    auth_finished(ap, AUTH_REJECT);
+	    return;
+	}
+	if (addr.sin_family != AF_INET) {
+	    if (auth_debug_mode)
+		printf("unknown address family: %d\r\n", addr.sin_family);
+	    Data(ap, KRB_REJECT, "bad address family", -1);
+	    auth_finished(ap, AUTH_REJECT);
+	    return;
+	}
+
+	r = krb_rd_req(&auth, KRB_SERVICE_NAME,
+		       instance, addr.sin_addr.s_addr, &adat, "");
+	if (r) {
 	    if (auth_debug_mode)
 		printf("Kerberos failed him as %s\r\n", name);
 	    Data(ap, KRB_REJECT, (void *)krb_get_err_text(r), -1);
@@ -322,7 +342,7 @@ kerberos4_is(Authenticator *ap, unsigned char *data, int cnt)
 		      krb_unparse_name_long(adat.pname, 
 					    adat.pinst, 
 					    adat.prealm), 
-		      UserNameRequested);
+		      UserNameRequested ? UserNameRequested : "<nobody>");
 	    if (msg == NULL)
 		Data(ap, KRB_REJECT, NULL, 0);
 	    else {
@@ -376,6 +396,7 @@ kerberos4_is(Authenticator *ap, unsigned char *data, int cnt)
 	    if(cnt > sizeof(cred))
 		abort();
 
+	    memcpy (session_key, adat.session, sizeof(session_key));
 	    des_set_key(&session_key, ks);
 	    des_pcbc_encrypt((void*)data, (void*)netcred, cnt, 
 			     ks, &session_key, DES_DECRYPT);
@@ -388,7 +409,7 @@ kerberos4_is(Authenticator *ap, unsigned char *data, int cnt)
 		   cred.issue_date < 0 || 
 		   cred.issue_date > time(0) + CLOCK_SKEW ||
 		   strncmp(cred.pname, adat.pname, sizeof(cred.pname)) ||
-		   strncmp(cred.pinst, adat.pinst, sizeof(cred.pname))){
+		   strncmp(cred.pinst, adat.pinst, sizeof(cred.pinst))){
 		    Data(ap, KRB_FORWARD_REJECT, "Bad credentials", -1);
 		}else{
 		    if((ret = tf_setup(&cred,
@@ -454,7 +475,7 @@ kerberos4_reply(Authenticator *ap, unsigned char *data, int cnt)
 	    skey.data = session_key;
 	    encrypt_session_key(&skey, 0);
 #if 0
-	    kerberos4_forward(ap);
+	    kerberos4_forward(ap, &cred_session);
 #endif
 	    return;
 	}
@@ -486,13 +507,13 @@ kerberos4_reply(Authenticator *ap, unsigned char *data, int cnt)
 }
 
 int
-kerberos4_status(Authenticator *ap, char *name, int level)
+kerberos4_status(Authenticator *ap, char *name, size_t name_sz, int level)
 {
     if (level < AUTH_USER)
 	return(level);
 
     if (UserNameRequested && !kuserok(&adat, UserNameRequested)) {
-	strcpy(name, UserNameRequested);
+	strcpy_truncate(name, UserNameRequested, name_sz);
 	return(AUTH_VALID);
     } else
 	return(AUTH_USER);
@@ -504,7 +525,6 @@ kerberos4_status(Authenticator *ap, char *name, int level)
 void
 kerberos4_printsub(unsigned char *data, int cnt, unsigned char *buf, int buflen)
 {
-    char lbuf[32];
     int i;
 
     buf[buflen-1] = '\0';		/* make sure its NULL terminated */
@@ -512,11 +532,11 @@ kerberos4_printsub(unsigned char *data, int cnt, unsigned char *buf, int buflen)
 
     switch(data[3]) {
     case KRB_REJECT:		/* Rejected (reason might follow) */
-	strncpy((char *)buf, " REJECT ", buflen);
+	strcpy_truncate((char *)buf, " REJECT ", buflen);
 	goto common;
 
     case KRB_ACCEPT:		/* Accepted (name might follow) */
-	strncpy((char *)buf, " ACCEPT ", buflen);
+	strcpy_truncate((char *)buf, " ACCEPT ", buflen);
     common:
 	BUMP(buf, buflen);
 	if (cnt <= 4)
@@ -529,25 +549,23 @@ kerberos4_printsub(unsigned char *data, int cnt, unsigned char *buf, int buflen)
 	break;
 
     case KRB_AUTH:			/* Authentication data follows */
-	strncpy((char *)buf, " AUTH", buflen);
+	strcpy_truncate((char *)buf, " AUTH", buflen);
 	goto common2;
 
     case KRB_CHALLENGE:
-	strncpy((char *)buf, " CHALLENGE", buflen);
+	strcpy_truncate((char *)buf, " CHALLENGE", buflen);
 	goto common2;
 
     case KRB_RESPONSE:
-	strncpy((char *)buf, " RESPONSE", buflen);
+	strcpy_truncate((char *)buf, " RESPONSE", buflen);
 	goto common2;
 
     default:
-	snprintf(lbuf, sizeof(lbuf), " %d (unknown)", data[3]);
-	strncpy((char *)buf, lbuf, buflen);
+	snprintf(buf, buflen, " %d (unknown)", data[3]);
     common2:
 	BUMP(buf, buflen);
 	for (i = 4; i < cnt; i++) {
-	    snprintf(lbuf, sizeof(lbuf), " %d", data[i]);
-	    strncpy((char *)buf, lbuf, buflen);
+	    snprintf(buf, buflen, " %d", data[i]);
 	    BUMP(buf, buflen);
 	}
 	break;
@@ -594,16 +612,25 @@ pack_cred(CREDENTIALS *cred, unsigned char *buf)
 {
     unsigned char *p = buf;
     
-    p += krb_put_nir(cred->service, cred->instance, cred->realm, p);
+    memcpy (p, cred->service, ANAME_SZ);
+    p += ANAME_SZ;
+    memcpy (p, cred->instance, INST_SZ);
+    p += INST_SZ;
+    memcpy (p, cred->realm, REALM_SZ);
+    p += REALM_SZ;
     memcpy(p, cred->session, 8);
     p += 8;
-    *p++ = cred->lifetime;
-    *p++ = cred->kvno;
-    p += krb_put_int(cred->ticket_st.length, p, 4);
+    p += KRB_PUT_INT(cred->lifetime, p, 4, 4);
+    p += KRB_PUT_INT(cred->kvno, p, 4, 4);
+    p += KRB_PUT_INT(cred->ticket_st.length, p, 4, 4);
     memcpy(p, cred->ticket_st.dat, cred->ticket_st.length);
     p += cred->ticket_st.length;
-    p += krb_put_int(cred->issue_date, p, 4);
-    p += krb_put_nir(cred->pname, cred->pinst, NULL, p);
+    p += KRB_PUT_INT(0, p, 4, 4);
+    p += KRB_PUT_INT(cred->issue_date, p, 4, 4);
+    memcpy (p, cred->pname, ANAME_SZ);
+    p += ANAME_SZ;
+    memcpy (p, cred->pinst, INST_SZ);
+    p += INST_SZ;
     return p - buf;
 }
 
@@ -611,24 +638,46 @@ static int
 unpack_cred(unsigned char *buf, int len, CREDENTIALS *cred)
 {
     unsigned char *p = buf;
+    u_int32_t tmp;
+
+    strncpy (cred->service, p, ANAME_SZ);
+    cred->service[ANAME_SZ - 1] = '\0';
+    p += ANAME_SZ;
+    strncpy (cred->instance, p, INST_SZ);
+    cred->instance[INST_SZ - 1] = '\0';
+    p += INST_SZ;
+    strncpy (cred->realm, p, REALM_SZ);
+    cred->realm[REALM_SZ - 1] = '\0';
+    p += REALM_SZ;
 
-    p += krb_get_nir(p, cred->service, cred->instance, cred->realm);
     memcpy(cred->session, p, 8);
     p += 8;
-    cred->lifetime = *p++;
-    cred->kvno = *p++;
+    p += krb_get_int(p, &tmp, 4, 0);
+    cred->lifetime = tmp;
+    p += krb_get_int(p, &tmp, 4, 0);
+    cred->kvno = tmp;
+
     p += krb_get_int(p, &cred->ticket_st.length, 4, 0);
     memcpy(cred->ticket_st.dat, p, cred->ticket_st.length);
+    p += cred->ticket_st.length;
+    p += krb_get_int(p, &tmp, 4, 0);
     cred->ticket_st.mbz = 0;
     p += krb_get_int(p, (u_int32_t *)&cred->issue_date, 4, 0);
-    p += krb_get_nir(p, cred->pname, cred->pinst, NULL);
+
+    strncpy (cred->pname, p, ANAME_SZ);
+    cred->pname[ANAME_SZ - 1] = '\0';
+    p += ANAME_SZ;
+    strncpy (cred->pinst, p, INST_SZ);
+    cred->pinst[INST_SZ - 1] = '\0';
+    p += INST_SZ;
     return 0;
 }
 
 
 int
-kerberos4_forward(Authenticator *ap)
+kerberos4_forward(Authenticator *ap, void *v)
 {
+    des_cblock *key = (des_cblock *)v;
     CREDENTIALS cred;
     char *realm;
     des_key_schedule ks;
@@ -646,10 +695,10 @@ kerberos4_forward(Authenticator *ap)
 		       &cred);
     if(ret)
 	return ret;
-    des_set_key(&session_key, ks);
+    des_set_key(key, ks);
     len = pack_cred(&cred, netcred);
     des_pcbc_encrypt((void*)netcred, (void*)netcred, len,
-		     ks, &session_key, DES_ENCRYPT);
+		     ks, key, DES_ENCRYPT);
     memset(ks, 0, sizeof(ks));
     Data(ap, KRB_FORWARD, netcred, len);
     memset(netcred, 0, sizeof(netcred));
