@@ -33,7 +33,7 @@
 
 #include "telnet_locl.h"
 
-RCSID("$Id: commands.c,v 1.33 1997/07/10 12:54:04 assar Exp $");
+RCSID("$Id: commands.c,v 1.48 1999/02/01 03:08:27 assar Exp $");
 
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
 int tos = -1;
@@ -66,7 +66,8 @@ makeargv()
     margc = 0;
     cp = line;
     if (*cp == '!') {		/* Special case shell escape */
-	strcpy(saveline, line);	/* save for shell command */
+	/* save for shell command */
+	strcpy_truncate(saveline, line, sizeof(saveline));
 	*argp++ = "!";		/* No room in string to get this */
 	margc++;
 	cp++;
@@ -1560,10 +1561,9 @@ env_init(void)
 
 		/* If this is not the full name, try to get it via DNS */
 		if (strchr(hbuf, '.') == 0) {
-			struct hostent *he = gethostbyname(hbuf);
-			if (he != 0)
-				strncpy(hbuf, he->h_name, 256);
-			hbuf[256] = '\0';
+			struct hostent *he = roken_gethostbyname(hbuf);
+			if (he != NULL)
+				strcpy_truncate(hbuf, he->h_name, 256);
 		}
 
 		asprintf (&cp, "%s%s", hbuf, cp2);
@@ -1602,7 +1602,7 @@ env_define(unsigned char *var, unsigned char *value)
 		if (ep->next)
 			ep->next->prev = ep;
 	}
-	ep->welldefined = opt_welldefined(var);
+	ep->welldefined = opt_welldefined((char *)var);
 	ep->export = 1;
 	ep->var = (unsigned char *)strdup((char *)var);
 	ep->value = (unsigned char *)strdup((char *)value);
@@ -1939,7 +1939,7 @@ status(int argc, char **argv)
  * Function that gets called when SIGINFO is received.
  */
 void
-ayt_status(void)
+ayt_status(int ignore)
 {
     call(status, "status", "notmuch", 0);
 }
@@ -1961,7 +1961,7 @@ cmdrc(char *m1, char *m2)
     if (skiprc)
 	return;
 
-    strcpy(m1save, m1);
+    strcpy_truncate(m1save, m1, sizeof(m1save));
     m1 = m1save;
 
     if (rcname[0] == 0) {
@@ -2029,18 +2029,25 @@ int
 tn(int argc, char **argv)
 {
     struct hostent *host = 0;
+#ifdef HAVE_IPV6
+    struct sockaddr_in6 sin6;
+#endif
     struct sockaddr_in sin;
+    struct sockaddr *sa = NULL;
+    int sa_size = 0;
     struct servent *sp = 0;
     unsigned long temp;
     extern char *inet_ntoa();
-#if	defined(IP_OPTIONS) && defined(IPPROTO_IP)
+#if defined(IP_OPTIONS) && defined(IPPROTO_IP)
     char *srp = 0;
     int srlen;
 #endif
-    char *cmd, *hostp = 0, *portp = 0, *user = 0;
+    char *cmd, *hostp = 0, *portp = 0;
+    char *user = 0;
+    int family, port = 0;
+    char **addr_list;
 
     /* clear the socket address prior to use */
-    memset(&sin, 0, sizeof(sin));
 
     if (connected) {
 	printf("?Already connected to %s\r\n", hostname);
@@ -2048,7 +2055,7 @@ tn(int argc, char **argv)
 	return 0;
     }
     if (argc < 2) {
-	strcpy(line, "open ");
+	strcpy_truncate(line, "open ", sizeof(line));
 	printf("(to) ");
 	fgets(&line[strlen(line)], sizeof(line) - strlen(line), stdin);
 	makeargv();
@@ -2064,7 +2071,7 @@ tn(int argc, char **argv)
 	    --argc; ++argv;
 	    if (argc == 0)
 		goto usage;
-	    user = *argv++;
+	    user = strdup(*argv++);
 	    --argc;
 	    continue;
 	}
@@ -2099,11 +2106,7 @@ tn(int argc, char **argv)
 	srp = 0;
 	temp = sourceroute(hostp, &srp, &srlen);
 	if (temp == 0) {
-#ifdef HAVE_H_ERRNO
 	    fprintf (stderr, "%s: %s\r\n", srp ? srp : "", hstrerror(h_errno));
-#else
-	    fprintf (stderr, "%s: %s\r\n", srp ? srp : "", "unknown error");
-#endif
 	    setuid(getuid());
 	    return 0;
 	} else if (temp == -1) {
@@ -2111,36 +2114,70 @@ tn(int argc, char **argv)
 	    setuid(getuid());
 	    return 0;
 	} else {
-	    sin.sin_addr.s_addr = temp;
-	    sin.sin_family = AF_INET;
+	    abort();
 	}
     } else {
+#endif
+	memset (&sin, 0, sizeof(sin));
+#ifdef HAVE_IPV6
+	memset (&sin6, 0, sizeof(sin6));
+
+	if(inet_pton(AF_INET6, hostp, &sin6.sin6_addr)) {
+	    sin6.sin6_family = family = AF_INET6;
+	    sa = (struct sockaddr *)&sin6;
+	    sa_size = sizeof(sin6);
+	    strcpy_truncate(_hostname, hostp, sizeof(_hostname));
+	    hostname =_hostname;
+	} else
 #endif
 	if(inet_aton(hostp, &sin.sin_addr)){
-	    sin.sin_family = AF_INET;
-	    strcpy(_hostname, hostp);
+	    sin.sin_family = family = AF_INET;
+	    sa = (struct sockaddr *)&sin;
+	    sa_size = sizeof(sin);
+	    strcpy_truncate(_hostname, hostp, sizeof(_hostname));
 	    hostname = _hostname;
 	} else {
-	    host = gethostbyname(hostp);
+#ifdef HAVE_GETHOSTBYNAME2
+#ifdef HAVE_IPV6
+	    host = gethostbyname2(hostp, AF_INET6);
+	    if(host == NULL)
+#endif
+		host = gethostbyname2(hostp, AF_INET);
+#else
+	    host = roken_gethostbyname(hostp);
+#endif
 	    if (host) {
-		sin.sin_family = host->h_addrtype;
-#if	defined(h_addr)		/* In 4.3, this is a #define */
-		memmove(&sin.sin_addr,
-			host->h_addr_list[0],
-			sizeof(sin.sin_addr));
-#else	/* defined(h_addr) */
-		memmove(&sin.sin_addr, host->h_addr, sizeof(sin.sin_addr));
-#endif	/* defined(h_addr) */
-		strncpy(_hostname, host->h_name, sizeof(_hostname));
+		strcpy_truncate(_hostname, host->h_name, sizeof(_hostname));
+		family = host->h_addrtype;
+		addr_list = host->h_addr_list;
+
+		switch(family) {
+		case AF_INET:
+		    memset(&sin, 0, sizeof(sin));
+		    sa_size = sizeof(sin);
+		    sa = (struct sockaddr *)&sin;
+		    sin.sin_family = family;
+		    sin.sin_addr   = *((struct in_addr *)(*addr_list));
+		    break;
+#ifdef HAVE_IPV6
+		case AF_INET6:
+		    memset(&sin6, 0, sizeof(sin6));
+		    sa_size = sizeof(sin6);
+		    sa = (struct sockaddr *)&sin6;
+		    sin6.sin6_family = family;
+		    sin6.sin6_addr   = *((struct in6_addr *)(*addr_list));
+		    break;
+#endif
+		default:
+		    fprintf(stderr, "Bad address family: %d\n", family);
+		    return 0;
+		}
+
 		_hostname[sizeof(_hostname)-1] = '\0';
 		hostname = _hostname;
 	    } else {
 	        fprintf (stderr, "%s: %s\r\n", hostp ? hostp : "",
-#ifdef HAVE_H_ERRNO
 			 hstrerror(h_errno));
-#else
-			 "unknown error");
-#endif
 		setuid(getuid());
 		return 0;
 	    }
@@ -2154,34 +2191,61 @@ tn(int argc, char **argv)
 	    telnetport = 1;
 	} else
 	    telnetport = 0;
-	sin.sin_port = atoi(portp);
-	if (sin.sin_port == 0) {
-	    sp = getservbyname(portp, "tcp");
+	port = atoi(portp);
+	if (port == 0) {
+	    sp = roken_getservbyname(portp, "tcp");
 	    if (sp)
-		sin.sin_port = sp->s_port;
+		port = sp->s_port;
 	    else {
 		printf("%s: bad port number\r\n", portp);
 		setuid(getuid());
 		return 0;
 	    }
 	} else {
-	    sin.sin_port = htons(sin.sin_port);
+	    port = htons(port);
 	}
     } else {
 	if (sp == 0) {
-	    sp = getservbyname("telnet", "tcp");
+	    sp = roken_getservbyname("telnet", "tcp");
 	    if (sp == 0) {
 		fprintf(stderr, "telnet: tcp/telnet: unknown service\r\n");
 		setuid(getuid());
 		return 0;
 	    }
-	    sin.sin_port = sp->s_port;
+	    port = sp->s_port;
 	}
 	telnetport = 1;
     }
-    printf("Trying %s...\r\n", inet_ntoa(sin.sin_addr));
     do {
-	net = socket(AF_INET, SOCK_STREAM, 0);
+	switch(family) {
+	case AF_INET:
+	    sin.sin_port = port;
+	    printf("Trying %s...\r\n", inet_ntoa(sin.sin_addr));
+	    break;
+#ifdef HAVE_IPV6
+	case AF_INET6: {
+#ifndef INET6_ADDRSTRLEN
+#define INET6_ADDRSTRLEN 46 
+#endif
+
+	    char buf[INET6_ADDRSTRLEN];
+
+	    sin6.sin6_port = port;
+#ifdef HAVE_INET_NTOP
+	    printf("Trying %s...\r\n", inet_ntop(AF_INET6,
+						 &sin6.sin6_addr,
+						 buf,
+						 sizeof(buf)));
+#endif
+	    break;
+	}
+#endif
+	default:
+	    abort();
+	}
+
+
+	net = socket(family, SOCK_STREAM, 0);
 	setuid(getuid());
 	if (net < 0) {
 	    perror("telnet: socket");
@@ -2194,7 +2258,7 @@ tn(int argc, char **argv)
 #endif
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
 	{
-# if	defined(HAS_GETTOS)
+# if	defined(HAVE_GETTOSBYNAME)
 	    struct tosent *tp;
 	    if (tos < 0 && (tp = gettosbyname("telnet", "tcp")))
 		tos = tp->t_tos;
@@ -2213,23 +2277,36 @@ tn(int argc, char **argv)
 		perror("setsockopt (SO_DEBUG)");
 	}
 
-	if (connect(net, (struct sockaddr *)&sin, sizeof (sin)) < 0) {
-#if	defined(h_addr)		/* In 4.3, this is a #define */
-	    if (host && host->h_addr_list[1]) {
+	if (connect(net, sa, sa_size) < 0) {
+	    if (host && addr_list[1]) {
 		int oerrno = errno;
 
-		fprintf(stderr, "telnet: connect to address %s: ",
-						inet_ntoa(sin.sin_addr));
+		switch(family) {
+		case AF_INET :
+		    fprintf(stderr, "telnet: connect to address %s: ",
+			    inet_ntoa(sin.sin_addr));
+		    sin.sin_addr = *((struct in_addr *)(*++addr_list));
+		    break;
+#ifdef HAVE_IPV6
+		case AF_INET6: {
+		    char buf[INET6_ADDRSTRLEN];
+
+		    fprintf(stderr, "telnet: connect to address %s: ",
+			    inet_ntop(AF_INET6, &sin6.sin6_addr, buf,
+				      sizeof(buf)));
+		    sin6.sin6_addr = *((struct in6_addr *)(*++addr_list));
+		    break;
+		}
+#endif
+		default:
+		    abort();
+		}
+		    
 		errno = oerrno;
 		perror(NULL);
-		host->h_addr_list++;
-		memmove(&sin.sin_addr,
-			host->h_addr_list[0],
-			sizeof(sin.sin_addr));
 		NetClose(net);
 		continue;
 	    }
-#endif	/* defined(h_addr) */
 	    perror("telnet: Unable to connect to remote host");
 	    return 0;
 	}
@@ -2244,7 +2321,7 @@ tn(int argc, char **argv)
 
 	user = getenv("USER");
 	if (user == NULL ||
-	    ((pw = k_getpwnam(user)) && pw->pw_uid != getuid())) {
+	    ((pw = k_getpwnam((char *)user)) && pw->pw_uid != getuid())) {
 		if ((pw = k_getpwuid(getuid())))
 			user = pw->pw_name;
 		else
@@ -2257,7 +2334,7 @@ tn(int argc, char **argv)
     }
     call(status, "status", "notmuch", 0);
     if (setjmp(peerdied) == 0)
-	telnet(user);
+	my_telnet((char *)user);
     NetClose(net);
     ExitString("Connection closed by foreign host.\r\n",1);
     /*NOTREACHED*/
@@ -2574,15 +2651,10 @@ sourceroute(char *arg, char **cpp, int *lenp)
 
 		if ((tmp = inet_addr(cp)) != -1) {
 			sin_addr.s_addr = tmp;
-		} else if ((host = gethostbyname(cp))) {
-#if	defined(h_addr)
+		} else if ((host = roken_gethostbyname(cp))) {
 			memmove(&sin_addr,
 				host->h_addr_list[0],
 				sizeof(sin_addr));
-#else
-			memmove(&sin_addr, host->h_addr,
-				sizeof(sin_addr));
-#endif
 		} else {
 			*cpp = cp;
 			return(0);
