@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997, 1998 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -38,23 +38,12 @@
 
 #include "krb5_locl.h"
 
-RCSID("$Id: cache.c,v 1.27 1997/07/16 21:24:38 joda Exp $");
-
-/* XXX shouldn't be here */
-
-void
-krb5_free_ccache(krb5_context context,
-		 krb5_ccache val)
-{
-    free(((krb5_fcache*)(val->data.data))->filename);
-    krb5_data_free (&val->data);
-    free(val);
-}
-
-extern krb5_cc_ops fcc_ops;
+RCSID("$Id: cache.c,v 1.36 1999/02/11 21:03:21 joda Exp $");
 
 krb5_error_code
-krb5_cc_register(krb5_context context, krb5_cc_ops *ops, int override)
+krb5_cc_register(krb5_context context, 
+		 const krb5_cc_ops *ops, 
+		 krb5_boolean override)
 {
     int i;
     if(context->cc_ops == NULL){
@@ -98,11 +87,13 @@ krb5_cc_resolve(krb5_context context,
     int i;
 
     if(context->cc_ops == NULL){
-	ret = krb5_cc_register(context, &fcc_ops, 1);
+	ret = krb5_cc_register(context, &krb5_fcc_ops, 1);
+	if(ret) return ret;
+	ret = krb5_cc_register(context, &krb5_mcc_ops, 1);
 	if(ret) return ret;
     }
 
-    for(i = 0; context->cc_ops[i].prefix && i < context->num_ops; i++)
+    for(i = 0; i < context->num_ops && context->cc_ops[i].prefix; i++)
 	if(strncmp(context->cc_ops[i].prefix, residual, 
 		   strlen(context->cc_ops[i].prefix)) == 0){
 	    krb5_ccache p;
@@ -119,22 +110,20 @@ krb5_cc_resolve(krb5_context context,
     return KRB5_CC_UNKNOWN_TYPE;
 }
 
-
-#if 0
 krb5_error_code
 krb5_cc_gen_new(krb5_context context,
-		krb5_cc_ops *ops,
+		const krb5_cc_ops *ops,
 		krb5_ccache *id)
 {
-}
+    krb5_ccache p;
 
-krb5_error_code
-krb5_cc_register(krb5_context context,
-		 krb5_cc_ops *ops,
-		 krb5_boolean override)
-{
+    p = malloc (sizeof(*p));
+    if (p == NULL)
+	return KRB5_CC_NOMEM;
+    p->ops = (krb5_cc_ops *)ops;
+    *id = p;
+    return p->ops->gen_new(context, id);
 }
-#endif
 
 char*
 krb5_cc_get_name(krb5_context context,
@@ -185,7 +174,11 @@ krb5_error_code
 krb5_cc_destroy(krb5_context context,
 		krb5_ccache id)
 {
-    return id->ops->destroy(context, id);
+    krb5_error_code ret;
+
+    ret = id->ops->destroy(context, id);
+    krb5_cc_close (context, id);
+    return ret;
 }
 
 krb5_error_code
@@ -194,9 +187,6 @@ krb5_cc_close(krb5_context context,
 {
     krb5_error_code ret;
     ret = id->ops->close(context, id);
-#if 0 /* XXX */
-    free(id->residual);
-#endif
     free(id);
     return ret;
 }
@@ -220,10 +210,11 @@ krb5_cc_retrieve_cred(krb5_context context,
     krb5_cc_cursor cursor;
     krb5_cc_start_seq_get(context, id, &cursor);
     while((ret = krb5_cc_next_cred(context, id, creds, &cursor)) == 0){
-	if(krb5_principal_compare(context, mcreds->server, creds->server)){
+	if(krb5_compare_creds(context, whichfields, mcreds, creds)){
 	    ret = 0;
 	    break;
 	}
+	krb5_free_creds_contents (context, creds);
     }
     krb5_cc_end_seq_get(context, id, &cursor);
     return ret;
@@ -279,3 +270,33 @@ krb5_cc_set_flags(krb5_context context,
     return id->ops->set_flags(context, id, flags);
 }
 		    
+krb5_error_code
+krb5_cc_copy_cache(krb5_context context,
+		   krb5_ccache from,
+		   krb5_ccache to)
+{
+    krb5_error_code ret;
+    krb5_cc_cursor cursor;
+    krb5_creds cred;
+    krb5_principal princ;
+    ret = krb5_cc_get_principal(context, from, &princ);
+    if(ret)
+	return ret;
+    ret = krb5_cc_initialize(context, to, princ);
+    if(ret){
+	krb5_free_principal(context, princ);
+	return ret;
+    }
+    ret = krb5_cc_start_seq_get(context, from, &cursor);
+    if(ret){
+	krb5_free_principal(context, princ);
+	return ret;
+    }
+    while(ret == 0 && krb5_cc_next_cred(context, from, &cred, &cursor) == 0){
+	ret = krb5_cc_store_cred(context, to, &cred);
+	krb5_free_creds_contents (context, &cred);
+    }
+    krb5_cc_end_seq_get(context, from, &cursor);
+    krb5_free_principal(context, princ);
+    return ret;
+}
