@@ -37,7 +37,7 @@
  */
 
 #include "krb5_locl.h"
-RCSID("$Id: config_file.c,v 1.25 1998/04/05 07:12:37 joda Exp $");
+RCSID("$Id: config_file.c,v 1.29 1998/08/13 20:40:10 assar Exp $");
 
 /*
  * Netinfo implementation from Luke Howard <lukeh@xedoc.com.au>
@@ -202,17 +202,17 @@ parse_section(char *p, krb5_config_section **s, krb5_config_section **parent)
     tmp = malloc(sizeof(*tmp));
     if (tmp == NULL)
 	return -1;
-    if (*s)
-	(*s)->next = tmp;
-    else
-	*parent = tmp;
-    *s = tmp;
     tmp->name = strdup(p+1);
     if (tmp->name == NULL)
 	return -1;
     tmp->type = krb5_config_list;
     tmp->u.list = NULL;
     tmp->next = NULL;
+    if (*s)
+	(*s)->next = tmp;
+    else
+	*parent = tmp;
+    *s = tmp;
     return 0;
 }
 
@@ -229,6 +229,8 @@ parse_list(FILE *f, unsigned *lineno, krb5_config_binding **parent)
 	if (buf[strlen(buf) - 1] == '\n')
 	    buf[strlen(buf) - 1] = '\0';
 	p = buf;
+	while(isspace(*p))
+	    ++p;
 	if (*p == '#' || *p == ';')
 	    continue;
 	while(isspace(*p))
@@ -248,7 +250,7 @@ parse_binding(FILE *f, unsigned *lineno, char *p,
 {
     krb5_config_binding *tmp;
     char *p1;
-    int ret;
+    int ret = 0;
 
     p1 = p;
     while (*p && !isspace(*p))
@@ -259,11 +261,6 @@ parse_binding(FILE *f, unsigned *lineno, char *p,
     tmp = malloc(sizeof(*tmp));
     if (tmp == NULL)
 	return -1;
-    if (*b)
-	(*b)->next = tmp;
-    else
-	*parent = tmp;
-    *b = tmp;
     tmp->name = strdup(p1);
     tmp->next = NULL;
     ++p;
@@ -278,13 +275,20 @@ parse_binding(FILE *f, unsigned *lineno, char *p,
 	tmp->type = krb5_config_list;
 	tmp->u.list = NULL;
 	ret = parse_list (f, lineno, &tmp->u.list);
-	if (ret)
-	    return ret;
     } else {
+	p1 = p;
+	while (*p && !isspace(*p))
+	    ++p;
+	*p = '\0';
 	tmp->type = krb5_config_string;
-	tmp->u.string = strdup(p);
+	tmp->u.string = strdup(p1);
     }
-    return 0;
+    if (*b)
+	(*b)->next = tmp;
+    else
+	*parent = tmp;
+    *b = tmp;
+    return ret;
 }
 
 krb5_error_code
@@ -309,10 +313,10 @@ krb5_config_parse_file (const char *fname, krb5_config_section **res)
 	if(buf[strlen(buf) - 1] == '\n')
 	    buf[strlen(buf) - 1] = '\0';
 	p = buf;
-	if (*p == '#' || *p == ';')
-	    continue;
 	while(isspace(*p))
 	    ++p;
+	if (*p == '#' || *p == ';')
+	    continue;
 	if (*p == '[') {
 	    ret = parse_section(p, &s, res);
 	    if (ret)
