@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997, 1998 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -38,27 +38,94 @@
 
 #include "krb5_locl.h"
 
-RCSID("$Id: context.c,v 1.12 1997/07/16 21:24:40 joda Exp $");
+RCSID("$Id: context.c,v 1.39 1999/02/11 21:03:23 joda Exp $");
+
+int issuid(void); /* XXX */
 
 krb5_error_code
 krb5_init_context(krb5_context *context)
 {
     krb5_context p;
-    const char *skew;
-    p = ALLOC(1, krb5_context_data);
+    int val;
+    const char *config_file = NULL;
+    const char * tmp;
+    krb5_config_section *tmp_cf;
+    krb5_error_code ret;
+
+    ALLOC(p, 1);
     if(!p)
 	return ENOMEM;
     memset(p, 0, sizeof(krb5_context_data));
+
+    /* init error tables */
     krb5_init_ets(p);
-    p->cc_ops = NULL;
-    krb5_config_parse_file (krb5_config_file, &p->cf);
+
+    if(!issuid())
+	config_file = getenv("KRB5_CONFIG");
+    if (config_file == NULL)
+	config_file = krb5_config_file;
+
+    ret = krb5_config_parse_file (config_file, &tmp_cf);
+
+    if (ret == 0)
+	p->cf = tmp_cf;
+    else
+	krb5_warnx (p, "Unable to parse config file %s.  Ignoring.",
+		    config_file); /* XXX */
+
     p->max_skew = 5 * 60;
-    skew = krb5_config_get_string (p->cf, "libdefaults", "clockskew", NULL);
-    if(skew){
-	int tmp;
-	if(sscanf(skew, "%d", &tmp) == 1)
-	    p->max_skew = tmp;
+    val = krb5_config_get_time (p, NULL, "libdefaults", "clockskew", NULL);
+    if (val >= 0)
+	p->max_skew = val;
+
+    p->kdc_timeout = 3;
+    val = krb5_config_get_time (p, NULL, "libdefaults", "kdc_timeout", NULL);
+    if(val >= 0) 
+	p->kdc_timeout = val;
+
+    p->max_retries = 3;
+    val = krb5_config_get_int (p, NULL, "libdefaults", "max_retries", NULL);
+    if (val >= 0)
+	p->max_retries = val;
+
+    p->http_proxy = krb5_config_get_string(p, NULL, "libdefaults", 
+					   "http_proxy", NULL);
+
+    {
+	char **etypes;
+	etypes = krb5_config_get_strings(p, NULL, "libdefaults", 
+					 "default_etypes", NULL);
+	if(etypes){
+	    int i, j, k;
+	    for(i = 0; etypes[i]; i++);
+	    p->etypes = malloc((i+1) * sizeof(*p->etypes));
+	    for(j = 0, k = 0; j < i; j++) {
+		if(krb5_string_to_enctype(p, etypes[j], &p->etypes[k]) == 0)
+		    k++;
+	    }
+	    p->etypes[k] = ETYPE_NULL;
+	    krb5_config_free_strings(etypes);
+	}
     }
+    /* default keytab name */
+    p->default_keytab = krb5_config_get_string(p, NULL, 
+					       "libdefaults", 
+					       "default_keytab_name", 
+					       NULL);
+    if(p->default_keytab == NULL)
+	p->default_keytab = KEYTAB_DEFAULT;
+
+    p->time_fmt = krb5_config_get_string(p, NULL, "libdefaults", 
+					 "time_format", NULL);
+    if(p->time_fmt == NULL)
+	p->time_fmt = "%d-%b-%Y %H:%M:%S";
+    p->log_utc = krb5_config_get_bool(p, NULL, "libdefaults", "log_utc", NULL);
+
+    /* init dns-proxy slime */
+    tmp = krb5_config_get_string(p, NULL, "libdefaults", 
+				 "dns_proxy", NULL);
+    if(tmp) 
+	roken_gethostby_setup(p->http_proxy, tmp);
     krb5_set_default_realm(p, NULL);
     *context = p;
     return 0;
@@ -79,56 +146,46 @@ krb5_free_context(krb5_context context)
   free(context);
 }
 
-
-/*
- * XXX - This information needs to be coordinated with encrypt.c
- */
-
-static krb5_boolean
-valid_etype(krb5_enctype e)
-{
-  return e == ETYPE_DES_CBC_CRC
-      || e == ETYPE_DES_CBC_MD4
-      || e == ETYPE_DES_CBC_MD5;
-}
-
 static krb5_error_code
 default_etypes(krb5_enctype **etype)
 {
-  krb5_enctype *p;
-  p = ALLOC(4, krb5_enctype);
-  if(!p)
-    return ENOMEM;
-  p[0] = ETYPE_DES_CBC_MD5;
-  p[1] = ETYPE_DES_CBC_MD4;
-  p[2] = ETYPE_DES_CBC_CRC;
-  p[3] = 0;
-  *etype = p;
-  return 0;
+    krb5_enctype p[] = {
+	ETYPE_DES3_CBC_SHA1,
+	ETYPE_DES3_CBC_MD5,
+	ETYPE_DES_CBC_MD5,
+	ETYPE_DES_CBC_MD4,
+	ETYPE_DES_CBC_CRC,
+	ETYPE_NULL
+    };
+    *etype = malloc(sizeof(p));
+    if(*etype == NULL)
+	return ENOMEM;
+    memcpy(*etype, p, sizeof(p));
+    return 0;
 }
 
 krb5_error_code
 krb5_set_default_in_tkt_etypes(krb5_context context, 
 			       const krb5_enctype *etypes)
 {
-  int i;
-  krb5_enctype *p = NULL;
+    int i;
+    krb5_enctype *p = NULL;
 
-  if(etypes) {
-    i = 0;
-    while(etypes[i])
-      if(!valid_etype(etypes[i++]))
-	return KRB5_PROG_ETYPE_NOSUPP;
-    ++i;
-    p = ALLOC(i, krb5_enctype);
-    if(!p)
-      return ENOMEM;
-    memmove(p, etypes, i * sizeof(krb5_enctype));
-  }
-  if(context->etypes)
-    free(context->etypes);
-  context->etypes = p;
-  return 0;
+    if(etypes) {
+	i = 0;
+	while(etypes[i])
+	    if(!krb5_enctype_valid(context, etypes[i++]))
+		return KRB5_PROG_ETYPE_NOSUPP;
+	++i;
+	ALLOC(p, i);
+	if(!p)
+	    return ENOMEM;
+	memmove(p, etypes, i * sizeof(krb5_enctype));
+    }
+    if(context->etypes)
+	free(context->etypes);
+    context->etypes = p;
+    return 0;
 }
 
 
@@ -143,7 +200,7 @@ krb5_get_default_in_tkt_etypes(krb5_context context,
   if(context->etypes) {
     for(i = 0; context->etypes[i]; i++);
     ++i;
-    p = ALLOC(i, krb5_enctype);
+    ALLOC(p, i);
     if(!p)
       return ENOMEM;
     memmove(p, context->etypes, i * sizeof(krb5_enctype));
@@ -167,12 +224,20 @@ void
 krb5_init_ets(krb5_context context)
 {
     if(context->et_list == NULL){
-	initialize_krb5_error_table(&context->et_list);
-#if 0
-	initialize_kv5m_error_table(&context->et_list);
-	initialize_kdb5_error_table(&context->et_list);
-#endif
-	initialize_asn1_error_table(&context->et_list);
-	initialize_hdb_error_table(&context->et_list);
+	initialize_krb5_error_table_r(&context->et_list);
+	initialize_asn1_error_table_r(&context->et_list);
+	initialize_heim_error_table_r(&context->et_list);
     }
 }
+
+void
+krb5_set_use_admin_kdc (krb5_context context, krb5_boolean flag)
+{
+    context->use_admin_kdc = flag;
+}
+
+krb5_boolean
+krb5_get_use_admin_kdc (krb5_context context)
+{
+    return context->use_admin_kdc;
+}
